@@ -11,7 +11,9 @@
 #include "appstream/AppStreamUtils.h"
 #include "config-paths.h"
 #include <AppStreamQt/spdx.h>
+#include <KIO/ApplicationLauncherJob>
 #include <KLocalizedString>
+#include <KService>
 #include <KShell>
 #include <PackageKit/Daemon>
 #include <QDebug>
@@ -329,33 +331,32 @@ void PackageKitResource::updateDetail(const QString &packageID,
     const QByteArray xx = _updateText.toUtf8();
     MMIOT *markdownHandle = mkd_string(xx.constData(), _updateText.size(), 0);
 
-    QString updateText;
     if (!mkd_compile(markdownHandle, MKD_FENCEDCODE | MKD_GITHUBTAGS | MKD_AUTOLINK)) {
-        updateText = _updateText;
+        m_changelog = _updateText;
     } else {
         char *htmlDocument;
         const int size = mkd_document(markdownHandle, &htmlDocument);
 
-        updateText = QString::fromUtf8(htmlDocument, size);
+        m_changelog = QString::fromUtf8(htmlDocument, size);
     }
     mkd_cleanup(markdownHandle);
 
 #else
-    const auto &updateText = _updateText;
+    m_changelog = _updateText;
 #endif
 
     const auto name = PackageKit::Daemon::packageName(packageID);
 
     QString info;
     addIfNotEmpty(i18n("Obsoletes:"), joinPackages(obsoletes, {}, name), info);
-    addIfNotEmpty(i18n("Release Notes:"), updateText, info);
+    addIfNotEmpty(i18n("Release Notes:"), changelog(), info);
     addIfNotEmpty(i18n("Update State:"), PackageKitMessages::updateStateMessage(state), info);
     addIfNotEmpty(i18n("Restart:"), PackageKitMessages::restartMessage(restart), info);
 
     if (!vendorUrls.isEmpty())
         addIfNotEmpty(i18n("Vendor:"), urlToLinks(vendorUrls).join(QLatin1String(", ")), info);
 
-    Q_EMIT changelogFetched(changelog() + info);
+    Q_EMIT changelogFetched(info);
 }
 
 PackageKitBackend *PackageKitResource::backend() const
@@ -431,18 +432,16 @@ bool PackageKitResource::extendsItself() const
     return true;
 }
 
-void PackageKitResource::runService(const QStringList &desktopFilePaths) const
+void PackageKitResource::runService(KService::Ptr service) const
 {
-    for (const QString &desktopFilePath : desktopFilePaths) {
-        auto p = new QProcess(parent());
-        connect(p, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [this, p](int exitCode, QProcess::ExitStatus /*exitStatus*/) {
-            if (exitCode != 0) {
-                Q_EMIT backend()->passiveMessage(i18n("Failed to start '%1'", KShell::joinArgs(p->arguments())));
-            }
-            p->deleteLater();
-        });
-        p->start(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice"), {desktopFilePath});
-    }
+    auto *job = new KIO::ApplicationLauncherJob(service);
+    connect(job, &KJob::finished, this, [this, service](KJob *job) {
+        if (job->error()) {
+            Q_EMIT backend()->passiveMessage(i18n("Failed to start '%1': %2", service->name(), job->errorString()));
+        }
+    });
+
+    job->start();
 }
 
 bool PackageKitResource::isCritical() const
